#include "ACL.H"
#include "AT898252.H"
#include "defines.h"
#include "1Wire.h"
#include "service.h"
#include "lcd.h"
#include "time.h"
#include "stdlib.h"
#include "stdio.h"

extern char iData[8];
extern char sz[17];
extern char cr;

extern char tm_sec;     /* seconds after the minute - [0,59] */
extern char tm_min;     /* minutes after the hour - [0,59] */
extern char tm_hour;    /* hours since midnight - [0,23] */
extern char tm_mday;    /* day of the month - [1,31] */
extern char tm_mon;     /* months since January - [1,12] */
extern int tm_year;    /* year */

// Writes data to EEMPROM at specified address
void WriteEEPROM(char xdata *pDest,char *pSrc, char iLen)
{
	char i=0;
	P1_0=0;

	WMCON&=~WDTEN_;

	WMCON=WMCON|EEMEN_|EEMWE_;	// Enable EEPROM access and Write mode

	while (i<iLen)
	{	
		pDest[i]=pSrc[i];		// Write byte
		while (!(WMCON&EERDY_));// Wait for the end of write cycle
		i++;
	}
	WMCON&=~EEMWE_;				// Disable write mode

	P1_0=1;
	WMCON|=WDTEN_|PS0_|PS1_|PS2_|WDTRST_;

}

// Clears EEPROM memory on Chip
void InitACLMemory(void)
{
	char xdata *pEEPROM;
	P1_0=0;
	pEEPROM=0x0;

	WMCON&=~WDTEN_;
	WMCON=WMCON|EEMEN_|EEMWE_;	// Enable EEPROM access and Write mode

	while (pEEPROM<=0x07FF)
	{	
		P1_1=0;
		*pEEPROM=0;				// Write byte
		while (!(WMCON&EERDY_));// Wait for the end of write cycle
		P1_1=1;

		pEEPROM++;
	}
	WMCON&=~EEMWE_;				// Disable write mode

	P1_0=1;
	WMCON|=WDTEN_|PS0_|PS1_|PS2_|WDTRST_;
}

// Searches for free space in EEPROM
char xdata *GetFreeSRSlot(void)
{
	unsigned char i=0;
	sr xdata *pSR=ACL_START;

	WMCON=WMCON|EEMEN_;

	// Look through memory slots and find free one
	while (i<MAXSR)
	{
		if (pSR->type==0 || pSR->type==0xFF)	// free slot
			return pSR;
		else
		{
			pSR++;
			i++;
		}
	}
	return NULL;
}

// Adds new security record (SR)
void AddSR(void)
{
	char type;
	long timelimit;

	sr xdata *pSlot;

	pSlot=GetFreeSRSlot();
	
	if (pSlot==NULL)
	{
		xmessagebox("nomem");
		return;
	}
	
	lcd_send_str("Press any key");
	lcd_send_str("Apply iButton");
	readkey();

	iData[0]=0;	// CRC position

	while (1)
	{
		if (OneWireReadSerial())
			break;
		if (getkey()>9)
			break;
	}
	
	if (iData[0]==0)	// CRC can not be 0 -> user has cancelled the action
		return;

onemoretry:	
	type=xmessagebox("1-Adm 2-Usr 3-Tmp Usr");

	if (type!=1 && type!=2 && type!=3)
		goto onemoretry;
	
	if (type==3)
		while (!ReadTime(&timelimit));
	

netudaNazhal:
	cr=xmessagebox("Save? 1/0");
	if (cr!=1 && cr!=0)
		goto netudaNazhal;
	
	if (cr==0)
		return;

	WriteEEPROM(&(pSlot->type),&type,1);// save type
	WriteEEPROM(&(pSlot->sid),&iData[1],7);	// iKey ID without CRC
	if (type==3)
		WriteEEPROM(&(pSlot->timeLimit),&timelimit,4);	// timelimit

	xmessagebox("Ok");
}

// Accounts Management
void ManageSR(void)
{
	unsigned char i=0;
	sr xdata *pSR=ACL_START;

	WMCON=WMCON|EEMEN_;

	xmessagebox("1Del <# *> 0Exit");

	while(1)
	{
		if (pSR[i].type)
		{
			lcd_clear();

			switch (pSR[i].type)
			{
				case 1:
					lcd_send_str("Adm");
					break;
				case 2:
					lcd_send_str("Usr");
					break;
				case 3:
					time2tm(pSR[i].timeLimit);
					sprintf(sz,"0%d/%bd/%bd %bd:%bd:%bd",tm_year-2000,tm_mon,tm_mday,tm_hour,tm_min,tm_sec);
					lcd_send_str(sz);
					break;
			}
			Byte2Hex (sz,pSR[i].sid,7);
			lcd_send_str(sz);
		}
		else
		{
			sprintf(sz,"%bu:Free",i);
			lcd_print(sz);
		}
		
		cr=readkey();
		switch (cr)
		{
			case 1:// delete record
				WMCON=WMCON|EEMEN_|EEMWE_;	// enable write mode
				pSR[i].type=0;			
				while (!(WMCON&EERDY_));	// Wait for the end of write cycle
				WMCON&=~EEMWE_;				// Disable write mode
				break;
				
			case 10:
				if (i>0)	// move left
					i--;
				else
					xmessagebox("Start");
				break;
			case 11:
				if (i<(MAXSR-1))
					i++;
				else
					xmessagebox("End");
				break;
			case 0:
				return;
		}
	}
}
