#include "time.h"
#include "1Wire.h"
#include "lcd.h"
#include "service.h"
#include "stdlib.h"
#include "stdio.h"


extern char sz[17];
extern char iData[8];
extern time_t internalTime;
extern char cr;

extern char tm_sec;     /* seconds after the minute - [0,59] */
extern char tm_min;     /* minutes after the hour - [0,59] */
extern char tm_hour;    /* hours since midnight - [0,23] */
extern char tm_mday;    /* day of the month - [1,31] */
extern char tm_mon;     /* months since January - [1,12] */
extern int tm_year;    /* year */

const char code dtTemplate[15]="YYYYMMDDHHMMSS";

void ClockOn(void)
{
	if (EnableClock(1))
		xmessagebox("Ok");
	else
		xmessagebox("Fail");
}

void ClockOff(void)
{
	if (EnableClock(0))
		xmessagebox("Ok");
	else
		xmessagebox("Fail");
}

char EnableClock(char enable)
{
	if (!OneWireReadMem  (0x0201,&cr,1))	// read contents of control register
		return 0;

	if (enable)
		cr|=(1<<4); // Oscillator control bit is on the 5-th position
	else
		cr&=~(1<<4);

	if (!OneWireWriteMem (0x0201,&cr,1))	//write new value to control register
		return 0;

	return 1;
}

// saves time in InternalTime global variable
void GetCurTime(void)
{
	char *p;

	if (!OneWireReadMem  (0x0203,iData,4))	// read contents of RTC memory. No partial secs.
	{	
		xmessagebox("RTCErr");
		internalTime=0;
		return;
	}
	
	p=&internalTime;
	p[0]=iData[3];
	p[1]=iData[2];
	p[2]=iData[1];
	p[3]=iData[0];
}

void ShowTime(void)
{
	unsigned char t;

	cr=0;

	do
	{
    	t=0x1F;
		while(t--)
 		  QueryTouch();

		GetCurTime();
		if (!internalTime)
			return;

		time2tm(internalTime);
		
		// this will refresh the screen only one time per second
		if (cr==tm_sec)
			continue;
		cr=tm_sec;

		sprintf(sz,"%d/%2bd/%2bd",tm_year,tm_mon,tm_mday);
		if (tm_mon<10)
			sz[5]='0';
		if (tm_mday<10)
			sz[8]='0';
		lcd_print(sz);

		sprintf(sz,"%2bd:%2bd:%2bd",tm_hour,tm_min,tm_sec);
		if (tm_hour<10)
			sz[0]='0';
		if (tm_min<10)
			sz[3]='0';
		if (tm_sec<10)
			sz[6]='0';

		lcd_send_str(sz);
	}while(getkey()==-1);
}


char ReadTime(time_t *internalTime)
{
	register char i=0;

	lcd_send_cmd(8|4|2|1);	// cursor & blink on |2 - cursor on, |1 - blink on	
	lcd_print(dtTemplate);
	lcd_send_cmd(2);			// return home
	
	do
	{
		cr=readkey();
		if (cr>9)
		{
			lcd_send_cmd(8|4);		// cursor and blink off
			return 0;
		}
		sz[i]='0'+cr;
		lcd_send_char(sz[i++]);
	}
	while (i<14);
	
	lcd_send_cmd(8|4);		// cursor and blink off

    tm_sec=atoc(&sz[12],2);
    tm_min=atoc(&sz[10],2);;
    tm_hour=atoc(&sz[8],2);;
    tm_mday=atoc(&sz[6],2);;
    tm_mon=atoc(&sz[4],2);;
	sz[4]=0;
    tm_year=atoi(sz);

	tm2time(internalTime);
	return 1;
}

void SetDateTime(void)
{
	char *p;

	if (!ReadTime(&internalTime))
		return;

	p=&internalTime;
	iData[3]=p[0];
	iData[2]=p[1];
	iData[1]=p[2];
	iData[0]=p[3];

	if (!OneWireWriteMem(0x0203,iData,4))
	{	
		xmessagebox("Err.");
		return;
	}
	xmessagebox("Ok");
	return;
}

